@@ -14,6 +14,7 @@ return $get_parent_config_cache = {
 	 'file' => $config{'dhcpd_conf'},
 	 'members' => &get_config(),
 	 'line' => -1,
+	 'fline' => -1,
 	 'eline' => $get_config_lines };
 }
 
@@ -291,6 +292,7 @@ $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
 }
 
 # opt_input(text, name, &config, default, size, units)
+# Returns HTML for an optional text field
 sub opt_input
 {
 local($v, $rv);
@@ -307,6 +309,7 @@ return $rv;
 }
 
 # save_opt(name, &func, &parent, [indent], [quote])
+# Saves an optional text field
 sub save_opt
 {
 local($dir);
