@@ -152,5 +152,20 @@ return !&has_command("rpc.nfsd") && !&has_command("nfsd") &&
        !&has_command("rpc.knfsd") ? 0 : 1;
 }
 
+sub restart_mountd
+{
+&system_logged("$config{'portmap_command'} >/dev/null 2>&1 </dev/null")
+	if ($config{'portmap_command'});
+local $temp = &tempname();
+local $rv = &system_logged("($config{'restart_command'}) </dev/null >$temp 2>&1");
+local $out = `cat $temp`;
+unlink($temp);
+if ($rv) {
+	# something went wrong.. return an error
+	return "<pre>$out</pre>";
+	}
+return undef;
+}
+
 1;
 
