@@ -6,7 +6,12 @@
 do '../web-lib.pl';
 &init_config();
 require '../ui-lib.pl';
-do "$gconfig{'os_type'}-lib.pl";
+if ($gconfig{'os_type'} =~ /^\S+\-linux$/) {
+	do "linux-lib.pl";
+	}
+else {
+	do "$gconfig{'os_type'}-lib.pl";
+	}
 &foreign_require("mount", "mount-lib.pl");
 %access = &get_module_acl();
 
@@ -34,7 +39,7 @@ return @rv;
 sub get_dump
 {
 local %dump;
-&read_file("$module_config_directory/$_[0].dump", \%dump);
+&read_file("$module_config_directory/$_[0].dump", \%dump) || return undef;
 $dump{'id'} = $_[0];
 return \%dump;
 }
@@ -178,5 +183,19 @@ print "<tr $tb> <td><b>$_[0]</b></td> </tr>\n";
 print "<tr $cb> <td><table width=100%>\n";
 }
 
+# dump_directories(&dump)
+sub dump_directories
+{
+if (!&multiple_directory_support($_[0]->{'fs'})) {
+	return $_[0]->{'dir'};
+	}
+elsif ($_[0]->{'tabs'}) {
+	return split(/\t+/, $_[0]->{'dir'});
+	}
+else {
+	return split(/\s+/, $_[0]->{'dir'});
+	}
+}
+
 1;
 
