@@ -9,7 +9,12 @@ push(@rv, "ufs") if (&has_command("ufsdump"));
 return @rv;
 }
 
-$multiple_directory_support = 1;
+# multiple_directory_support(fs)
+# Returns 1 if some filesystem dump supports multiple directories
+sub multiple_directory_support
+{
+return $_[0] eq "ufs";
+}
 
 # dump_form(&dump)
 sub dump_form
