@@ -225,5 +225,28 @@ for($i=0; $i<@vsp || $i<@_; $i++) {
 return 1;	# same!
 }
 
+# apply_configuration()
+# Apply the heartbeat configuration, and return undef on success or an error
+# message on failure
+sub apply_configuration
+{
+if ($config{'apply_cmd'}) {
+	$out = &backquote_logged("$config{'apply_cmd'} 2>&1 </dev/null");
+	if ($?) {
+		return $out;
+		}
+	}
+else {
+	local $pid = &check_pid_file($config{'pid_file'});
+	if ($pid) {
+		kill(HUP, $pid);
+		}
+	else {
+		return $text{'apply_epid'};
+		}
+	}
+return undef;
+}
+
 1;
 
