@@ -165,6 +165,34 @@ ${$_[0]} = $out;
 return $out =~ /\s(1\.4\S*)/ ? $1 : undef;
 }
 
+# stop_jabber()
+# Stops jabber, and returns undef on success or an error message on failure
+sub stop_jabber
+{
+if ($config{'stop_cmd'}) {
+        &system_logged("$config{'stop_cmd'} </dev/null >/dev/null 2>&1");
+        }
+else {
+	local $pid = &check_pid_file(&jabber_pid_file());
+	if ($pid) {
+		&kill_logged('TERM', $pid) || return $!;
+		}
+	else {
+		return $text{'stop_epid'};
+		}
+        }
+unlink(&jabber_pid_file());
+return undef;
+}
+
+# start_jabber()
+# Starts jabber, and returns undef on success or an error message on failure
+sub start_jabber
+{
+&system_logged("$config{'start_cmd'} </dev/null >/tmp/err 2>&1");
+return undef;
+}
+
 @register_fields = ( 'name', 'email' );
 
 @karma_presets = ( { 'heartbeat' => 2,	'init' => 10,
