@@ -525,3 +525,28 @@ return { 'driver' => $drv->{'desc'},
 		   &dev_name($_[0]->{'dev'}) };
 }
 
+# parse_cups_ppd(file)
+# Converts a CUPS-style .ppd file into a hash of names and values
+sub parse_cups_ppd
+{
+local %ppd;
+if ($_[0] =~ /\.gz$/) {
+	open(PPD, "gunzip -c '$_[0]' |");
+	}
+else {
+	open(PPD, $_[0]);
+	}
+while(<PPD>) {
+	if (/^\s*\*(\S+):\s*"(.*)"/ || /^\s*\*(\S+):\s*(\S+)/) {
+		$ppd{$1} = $2;
+		}
+	elsif (/^\s*\*(\S+)\s+(\S+)\/([^:]+):/) {
+		$ppd{$1}->{$2} = $3 if (!defined($ppd{$1}->{$2}));
+		}
+	}
+close(PPD);
+return \%ppd;
+}
+
+1;
+
