@@ -1,6 +1,19 @@
 # solaris-lib.pl
 # Functions for solaris/hpux-style printer management
 
+# Find lpsched command
+# But first define the default.
+$config{'lpschedcmd'} = "/usr/lib/lp/lpsched";
+if (-x "/usr/lib/lpsched") {
+	$config{'lpschedcmd'} = "/usr/lib/lpsched";
+}
+
+$interface_dir = "/usr/lib/lp/model";
+$default_interface = "$interface_dir/standard";
+$netstandard_interface = "$interface_dir/netstandard";
+$foomatic_interface = "$interface_dir/standard_foomatic";
+$foomatic_netstandard_interface = "$interface_dir/netstandard_foomatic";
+
 # list_printers()
 # Returns an array of known printer names
 sub list_printers
@@ -55,7 +68,7 @@ if (!$prn{'enabled'} && $body =~ /^\s+(.*)/) {
 	}
 if ($body =~ /Description: (.*)/) { $prn{'desc'} = $1; }
 if ($body =~ /Printer types: (.*)/) { $prn{'ptype'} = $1; }
-if ($body =~ /Interface: (.*)/ && $1 ne $config{'default_interface'})
+if ($body =~ /Interface: (.*)/ && $1 ne $default_interface)
 	{ $prn{'iface'} = $1; }
 if ($body =~ /Banner not required/) { $prn{'banner'} = 0; }
 else { $prn{'banner'} = 1; }
@@ -70,6 +83,17 @@ if ($body =~ /Users (allowed|denied):\n((\s+\S+\n)+)/) {
 		}
 	else { $prn{'deny'} = \@l; }
 	}
+if ($body =~ /Options:\s*(.*)/) {
+	local $opts = $1;
+	local $o;
+	foreach $o (split(/,\s*/, $opts)) {
+		local ($on, $ov) = split(/=/, $o);
+		$prn{'options'}->{$on} = $ov;
+		}
+	}
+if ($body =~ /PPD:\s+(\S+)/ && $1 ne "none" && $1 ne "/dev/null") {
+	$prn{'ppd'} = $1;
+	}
 
 if (!$_[1]) {
 	# request availability
@@ -86,9 +110,25 @@ if (!$_[1]) {
 
 # request connection
 $con = `lpstat -v $esc 2>&1`;
-if ($con =~ /^device for \S+:\s+(\S+)/) { $prn{'dev'} = $1; }
+if ($con =~ /^device for \S+:\s+(\S+)/) {
+	# Prints to a local file
+	$prn{'dev'} = $1;
+	if ($prn{'dev'} eq "/dev/null" &&
+	    ($prn{'iface'} eq $netstandard_interface ||
+	     $prn{'iface'} eq $foomatic_netstandard_interface)) {
+		# Actually a remote TCP printer
+		local ($dh, $dp) = split(/:/, $prn{'options'}->{'dest'});
+		if ($dh) {
+			$prn{'dhost'} = $dh;
+			$prn{'dport'} = $dp;
+			delete($prn{'dev'});
+			delete($prn{'iface'});
+			}
+		}
+	}
 elsif ($con =~ /^system for \S+:\s+(\S+)\s+\(as printer (\S+)\)/ ||
        $con =~ /^system for \S+:\s+(\S+)/) {
+	# Prints to a remote server
 	$prn{'rhost'} = $1;
 	$prn{'rqueue'} = $2 || $prn{'name'};
 	$sys = `lpsystem -l $prn{'rhost'} 2>&1`;
@@ -164,7 +204,7 @@ return @jobs;
 # printer_support(option)
 sub printer_support
 {
-return $_[0] !~ /^(msize|alias|riface|direct|rnoqueue|ipp)$/;
+return $_[0] !~ /^(msize|alias|riface|rnoqueue|ipp)$/;
 }
 
 # list_classes()
@@ -193,6 +233,7 @@ sub modify_printer
 {
 local(%prn, $cmd, $out);
 %prn = %{$_[0]};
+local $old = &get_printer($prn{'name'});
 
 # call lpsystem if needed
 local $tesc = quotemeta($prn{'rtype'});
@@ -219,6 +260,7 @@ elsif ($prn{'deny'}) {
 	$cmd .= " -u deny:".join(',', map { quotemeta($_) } @{$prn{'deny'}});
 	}
 if ($prn{'dev'}) {
+	# Just printing to a device file
 	local $vesc = quotemeta($prn{'dev'});
 	$cmd .= " -v $vesc";
 	if ($prn{'iface'}) {
@@ -226,15 +268,51 @@ if ($prn{'dev'}) {
 		$cmd .= " -i $iesc";
 		}
 	else {
-		$cmd .= " -i \"$config{'default_interface'}\"";
+		if ($prn{'ppd'}) {
+			$cmd .= " -i $foomatic_interface";
+			}
+		else {
+			$cmd .= " -i $default_interface";
+			}
 		}
 	if ($prn{'banner'}) { $cmd .= " -o banner"; }
 	else { $cmd .= " -o nobanner"; }
 	}
-else { $cmd .= " -s $resc!$qesc"; }
-@ctype = @{$prn{'ctype'}};
-if (@ctype) { $cmd .= " -I ".join(',' , @ctype); }
-$out = &backquote_logged("cd / ; $cmd 2>&1");
+elsif ($prn{'dhost'}) {
+	# Printing to a remote host
+	local $hesc = quotemeta($prn{'dhost'});
+	local $pesc = quotemeta($prn{'dport'});
+	$cmd .= " -v /dev/null -o dest=$hesc:$pesc -o protocol=tcp";
+	if ($prn{'ppd'}) {
+		$cmd .= " -i $foomatic_netstandard_interface";
+		}
+	else {
+		$cmd .= " -i $netstandard_interface";
+		}
+	if ($prn{'banner'}) { $cmd .= " -o banner"; }
+	else { $cmd .= " -o nobanner"; }
+	}
+else {
+	# Printing to remote LPR server
+	$cmd .= " -s $resc!$qesc";
+	}
+
+# Add any content types
+local @ctype = @{$prn{'ctype'}};
+if (@ctype) {
+	$cmd .= " -I ".join(',' , @ctype);
+	}
+
+# Add PPD option
+if ($_[0]->{'ppd'}) {
+	$cmd .= " -n ".quotemeta($_[0]->{'ppd'});
+	}
+elsif ($old->{'ppd'}) {
+	# Need to clear out PPD .. but how?
+	$cmd .= " -n /dev/null";
+	}
+
+local $out = &backquote_logged("cd / ; $cmd 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out ($cmd)</pre>"); }
 
 # make the default
@@ -243,6 +321,14 @@ if ($prn{'default'}) {
 	if ($?) { &error("Failed to set default : <pre>$out</pre>"); }
 	}
 
+# Build filter table 
+open(STAT, "/usr/bin/ls -1 /etc/lp/fd/*.fd |");
+while(<STAT>) {
+	$file = substr($_, rindex($_, "/") +1, -4 );
+	`/usr/sbin/lpfilter -f $file -F /etc/lp/fd/$file.fd`;
+        }
+close(STAT);
+
 # enable or disable
 if ($prn{'enabled'}) { $cmd = "enable $esc"; }
 elsif ($prn{'enabled_why'}) {
@@ -305,7 +391,7 @@ return 0;
 # Start lpsched
 sub start_sched
 {
-local $out = &backquote_logged("/usr/lib/lp/lpsched 2>&1");
+local $out = &backquote_logged("$config{'lpschedcmd'} 2>&1");
 if ($?) { &error("failed to start lpsched : <tt>$out</tt>"); }
 }
 
@@ -329,14 +415,68 @@ return "lp -d $esc $fesc";
 # check_print_system()
 sub check_print_system
 {
+local $lpschedcmd = $config{'lpschedcmd'};
 &has_command("lpstat") || return &text('solaris_ecmd', "<tt>lpstat</tt>");
-&has_command("/usr/lib/lp/lpsched") || return &text('solaris_ecmd', "<tt>lpsched</tt>");
+&has_command("$lpschedcmd") || return &text('solaris_ecmd', "<tt>$lpschedcmd</tt>");
 return undef;
 }
 
-@device_files = ("/dev/bpp0", "/dev/ecpp0", "/dev/term/a",
-		 "/dev/term/b", "/dev/null");
-@device_names = ($text{'solaris_paralel'}, $text{'solaris_uparalel'},
-		 &text('solaris_serial', "A"), &text('solaris_serial', "B"),
-		 $text{'solaris_null'});
+#
+# get_device_files()
+#
+sub get_device_files
+{
+
+local @files;
+#
+# There are a string of files that could be used here, include only the
+# ones that actually exist.  Start with the parallel ports, and then the
+# term ports, lp and printers/*.
+
+$devlist = "/dev/bpp*";
+$devlist .= " /dev/ecpp*";
+$devlist .= " /dev/term/*";
+$devlist .= " /dev/lp*";
+$devlist .= " /dev/printers/*";
+open(DEV, "/bin/find $devlist -print 2> /dev/null |");
+while(<DEV>) {
+        push (@files, $_);
+	}
+close(DEV);
+#
+# And also include /dev/null
+#
+push (@files, "/dev/null");
+return @files;
+
+}
+
+#
+# get_device_names(@files)
+#
+sub get_device_names
+{
+local @files = @_;
+local @names;
+#
+# There are a string of files that could be used here, include only the
+# ones that actually exist.  Start with the parallel ports, and then the
+# term ports, lp and printers/*.
+
+for (@files) {
+	if (/bpp/) { push(@names, "$text{'solaris_paralel'} - $_"); }
+	elsif (/ecpp/) { push(@names, "$text{'solaris_paralel'} - $_"); }
+	elsif (/term\/a/) { push(@names, &text('solaris_serial', 'A') . " - $_"); }
+	elsif (/term\/b/) { push(@names, &text('solaris_serial', 'B') . " - $_"); }
+	elsif (/null/) { push(@names, "$text{'solaris_null'} - $_"); }
+	else { push(@names, "$_"); }
+}
+
+return @names;
+
+}
+
+@device_files = &get_device_files();
+@device_names = &get_device_names(@device_files);
+
 
