@@ -203,32 +203,88 @@ elsif ($_[2]->{'type'} == 4) {
 			}
 		}
 
+	return @mail;
+	}
+elsif ($_[2]->{'type'} == 5) {
+	# Just all of the constituent folders
+	local @mail;
+
+	# Work out exactly how big the total is
+	local ($sf, %len, $count);
+	foreach $sf (@{$_[2]->{'subfolders'}}) {
+		$len{$sf} = &mailbox_folder_size($sf);
+		$count += $len{$sf};
+		}
+
+	# Work out what range we need
+	local ($start, $end);
+	if (!defined($_[0])) {
+		$start = 0; $end = $count-1;
+		}
+	elsif ($_[1] < 0) {
+		$start = $count+$_[1]-1; $end = $count+$_[0]-1;
+		$start = $start<0 ? 0 : $start;
+		}
+	else {
+		$start = $_[0]; $end = $_[1];
+		$end = $count-1 if ($end >= $count);
+		}
+
+	# Fetch the needed part of each sub-folder
+	local $pos = 0;
+	foreach $sf (@{$_[2]->{'subfolders'}}) {
+		local ($sfstart, $sfend);
+		$sfstart = $start - $pos;
+		$sfend = $end - $pos;
+		$sfstart = $sfstart < 0 ? 0 :
+			   $sfstart >= $len{$sf} ? $len{$sf}-1 : $sfstart;
+		$sfend = $sfend < 0 ? 0 :
+			 $sfend >= $len{$sf} ? $len{$sf}-1 : $sfend;
+		local @submail =
+			&mailbox_list_mails($sfstart, $sfend, $sf, $_[3]);
+		local $sm;
+		foreach $sm (@submail) {
+			if ($sm) {
+				$sm->{'subidx'} = $sm->{'idx'};
+				$sm->{'idx'} += $pos;
+				$sm->{'subfolder'} = $sf;
+				}
+			}
+		push(@mail, @submail);
+		$pos += $len{$sf};
+		}
+
 	return @mail;
 	}
 }
 
-# mailbox_search_mail(&fields, andmode, &folder)
+# mailbox_search_mail(&fields, andmode, &folder, [&limit])
 sub mailbox_search_mail
 {
 if ($_[2]->{'type'} == 0) {
 	# Just search an mbox format file
-	return &advanced_search_mail($_[2]->{'file'}, $_[0], $_[1]);
+	return &advanced_search_mail($_[2]->{'file'}, $_[0], $_[1], $_[3]);
 	}
 elsif ($_[2]->{'type'} == 1) {
 	# Search a maildir directory
 	local $md = $_[2]->{'file'};
-	return &advanced_search_maildir($md, $_[0], $_[1]);
+	return &advanced_search_maildir($md, $_[0], $_[1], $_[3]);
 	}
 elsif ($_[2]->{'type'} == 2) {
 	# Get all of the mail from the POP3 server and search it
-	local @mails = &mailbox_list_mails(undef, undef, $_[2],
+	local ($min, $max);
+	if ($_[3] && $_[3]->{'latest'}) {
+		$min = -1;
+		$max = -$_[3]->{'latest'};
+		}
+	local @mails = &mailbox_list_mails($min, $max, $_[2],
 		   &indexof('body', &search_fields($_[0])) >= 0 ? 0 : 1);
-	local @rv = grep { &mail_matches($_[0], $_[1], $_) } @mails;
+	local @rv = grep { $_ && &mail_matches($_[0], $_[1], $_) } @mails;
 	}
 elsif ($_[2]->{'type'} == 3) {
 	# Search an MH directory
 	local $md = $_[2]->{'file'};
-	return &advanced_search_mhdir($md, $_[0], $_[1]);
+	return &advanced_search_mhdir($md, $_[0], $_[1], $_[3]);
 	}
 elsif ($_[2]->{'type'} == 4) {
 	# Use IMAP's remote search feature
@@ -273,17 +329,51 @@ elsif ($_[2]->{'type'} == 4) {
 	shift(@ids); shift(@ids);	# lose * SEARCH
 	local (@mail, $idx);
 	foreach $idx (@ids) {
+		local $realidx = $idx-1;
+		if ($_[3] && $_[3]->{'latest'}) {
+			# Don't get message if outside search range
+			next if ($realidx < $rv[3]-$_[3]->{'latest'});
+			}
 		local @rv = &imap_command($h,
 			"FETCH $idx (RFC822.SIZE RFC822.HEADER)");
 		&error(&text('save_esearch', $rv[3])) if (!$rv[0]); 
 		local $mail = &parse_imap_mail($rv[1]->[0]);
 		if ($mail) {
-			$mail->{'idx'} = $idx-1;
+			$mail->{'idx'} = $realidx;
 			push(@mail, $mail);
 			}
 		}
 	return reverse(@mail);
 	}
+elsif ($_[2]->{'type'} == 5) {
+	# Search each sub-folder and combine the results - taking any count
+	# limits into effect
+	local $sf;
+	local $pos = 0;
+	local @mail;
+	local (%start, %len);
+	foreach $sf (@{$_[2]->{'subfolders'}}) {
+		$len{$sf} = &mailbox_folder_size($sf);
+		$start{$sf} = $pos;
+		$pos += $len{$sf};
+		}
+	local $limit = $_[3] ? { %{$_[3]} } : undef;
+	foreach $sf (reverse(@{$_[2]->{'subfolders'}})) {
+		local @submail = &mailbox_search_mail($_[0], $_[1], $sf, $limit);
+		foreach $sm (@submail) {
+			$sm->{'subidx'} = $sm->{'idx'};
+			$sm->{'idx'} += $start{$sf};
+			$sm->{'subfolder'} = $sf;
+			}
+		push(@mail, reverse(@submail));
+		if ($limit && $limit->{'latest'}) {
+			# Adjust latest down by size of this folder
+			$limit->{'latest'} -= $len{$sf};
+			last if ($limit->{'latest'} <= 0);
+			}
+		}
+	return reverse(@mail);
+	}
 }
 
 # mailbox_delete_mail(&folder, mail, ...)
@@ -345,6 +435,16 @@ elsif ($f->{'type'} == 4) {
 	@rv = &imap_command($h, "EXPUNGE");
 	&error(&text('save_edelete', $rv[3])) if (!$rv[0]); 
 	}
+elsif ($f->{'type'} == 5) {
+	# Delete from underlying folder(s)
+	local $sm;
+	foreach $sm (@_) {
+		local $oldidx = $sm->{'idx'};
+		$sm->{'idx'} = $sm->{'subidx'};
+		&mailbox_delete_mail($sm->{'subfolder'}, $sm);
+		$sm->{'idx'} = $oldidx;
+		}
+	}
 }
 
 # mailbox_empty_folder(&folder)
@@ -395,6 +495,13 @@ elsif ($f->{'type'} == 4) {
 	@rv = &imap_command($h, "EXPUNGE");
 	&error(&text('save_edelete', $rv[3])) if (!$rv[0]); 
 	}
+elsif ($f->{'type'} == 5) {
+	# Empty each sub-folder
+	local $sf;
+	foreach $sf (@{$f->{'subfolders'}}) {
+		&mailbox_empty_folder($sf);
+		}
+	}
 }
 
 # mailbox_move_mail(&source, &dest, mail, ...)
@@ -494,7 +601,7 @@ elsif ($_[1]->{'type'} == 4) {
 	if ($rv[0] == 0) { &error($rv[1]); }
 	elsif ($rv[0] == 3) { &error(&text('save_emailbox', $rv[1])); }
 	elsif ($rv[0] == 2) { &error(&text('save_elogin2', $rv[1])); }
-local $h = $rv[1];
+	local $h = $rv[1];
 
 	# Create a temp file and use it to create the IMAP command
 	local $temp = &tempname();
@@ -508,6 +615,11 @@ local $h = $rv[1];
 			$_[1]->{'mailbox'} || "INBOX", length($text), $text);
 	&error(&text('save_eappend', $rv[3])) if (!$rv[0]); 
 	}
+elsif ($_[1]->{'type'} == 5) {
+	# Just append to the last subfolder
+	local @sf = @{$_[1]->{'subfolders'}};
+	&write_mail_folder($_[0], $sf[$#sf], $_[2]);
+	}
 }
 
 # mailbox_modify_mail(&oldmail, &newmail, &folder, textonly)
@@ -522,10 +634,67 @@ elsif ($_[2]->{'type'} == 3) {
 	# Just replace the existing file
 	&modify_mhdir($_[0], $_[1], $_[3]);
 	}
-else {
+elsif ($_[2]->{'type'} == 0) {
 	# Modify the mail file
 	&modify_mail($_[2]->{'file'}, $_[0], $_[1], $_[3]);
 	}
+elsif ($_[2]->{'type'} == 5) {
+	# Modify in the appropriate folder
+	local $oldoldidx = $_[0]->{'idx'};
+	$_[0]->{'idx'} = $_[0]->{'subidx'};
+	local $oldnewidx = $_[1]->{'idx'};
+	$_[1]->{'idx'} = $_[1]->{'subidx'};
+	&mailbox_modify_mail($_[0], $_[1], $_[0]->{'subfolder'}, $_[3]);
+	$_[0]->{'idx'} = $oldoldidx;
+	$_[1]->{'idx'} = $oldnewidx;
+	}
+else {
+	&error("Cannot modify mail in this type of folder!");
+	}
+}
+
+# mailbox_folder_size(&folder)
+# Returns the number of messages in some folder
+sub mailbox_folder_size
+{
+if ($_[0]->{'type'} == 0) {
+	# A mbox formatted file
+	return &count_mail($_[0]->{'file'});
+	}
+elsif ($_[0]->{'type'} == 1) {
+	# A qmail maildir
+	return &count_maildir($_[0]->{'file'});
+	}
+elsif ($_[0]->{'type'} == 2) {
+	# A POP3 server
+	local @rv = &pop3_login($_[0]);
+	if ($rv[0] != 1) {
+		if ($rv[0] == 0) { &error($rv[1]); }
+		else { &error(&text('save_elogin', $rv[1])); }
+		}
+	local @st = &pop3_command($rv[1], "stat");
+	if ($st[0] == 1) {
+		local ($count, $size) = split(/\s+/, $st[1]);
+		return $count;
+		}
+	else {
+		&error($st[1]);
+		}
+	}
+elsif ($_[0]->{'type'} == 3) {
+	# An MH directory
+	return &count_mhdir($_[0]->{'file'});
+	}
+elsif ($_[0]->{'type'} == 4) {
+	# An IMAP server
+	local @rv = &imap_login($_[0]);
+	if ($rv[0] != 1) {
+		if ($rv[0] == 0) { &error($rv[1]); }
+		elsif ($rv[0] == 3) { &error(&text('save_emailbox', $rv[1])); }
+		elsif ($rv[0] == 2) { &error(&text('save_elogin2', $rv[1])); }
+		}
+	return $rv[2];
+	}
 }
 
 # pop3_login(&folder)
@@ -993,10 +1162,12 @@ sub folder_size
 local ($f, $total);
 foreach $f (@_) {
 	if ($f->{'type'} == 0) {
+		# Single mail file - size is easy
 		local @st = stat($f->{'file'});
 		$f->{'size'} = $st[7];
 		}
 	elsif ($f->{'type'} == 1) {
+		# Maildir folder size is that of all mail files
 		local $qd;
 		$f->{'size'} = 0;
 		foreach $qd ('cur', 'new') {
@@ -1010,6 +1181,7 @@ foreach $f (@_) {
 			}
 		}
 	elsif ($f->{'type'} == 3) {
+		# MH folder size is that of all mail files
 		local $mf;
 		$f->{'size'} = 0;
 		opendir(MHDIR, $f->{'file'});
@@ -1019,7 +1191,12 @@ foreach $f (@_) {
 			}
 		closedir(MHDIR);
 		}
+	elsif ($f->{'type'} == 5) {
+		# Size of a combined folder is the size of all sub-folders
+		return &folder_size(@{$f->{'subfolders'}});
+		}
 	else {
+		# Cannot get size of a remote folder
 		$f->{'size'} = undef;
 		}
 	$total += $f->{'size'};
@@ -1073,7 +1250,9 @@ opendir(DIR, $_[0]);
 local @files = readdir(DIR);
 closedir(DIR);
 foreach $f (@files) {
-	next if ($f eq "." || $f eq ".." || $f =~ /\.lock$/i || $f eq "cur" || $f eq "tmp" || $f eq "new");
+	next if ($f eq "." || $f eq ".." || $f =~ /\.lock$/i ||
+		 $f eq "cur" || $f eq "tmp" || $f eq "new" ||
+		 $f =~ /^\.imap/i);
 	local $p = "$_[0]/$f";
 	if ($_[1] || !-d $p || -d "$p/cur") {
 		push(@rv, $p);
@@ -1099,7 +1278,7 @@ sub fix_cids
 local $rv = $_[0];
 $rv =~ s/(src="|href=")cid:([^"]+)(")/$1.&fix_cid($2,$_[1],$_[2]).$3/gei;
 $rv =~ s/(src='|href=')cid:([^']+)(')/$1.&fix_cid($2,$_[1],$_[2]).$3/gei;
-$rv =~ s/(src=|href=)cid:(\S+)()/$1.&fix_cid($2,$_[1],$_[2]).$3/gei;
+$rv =~ s/(src=|href=)cid:([^\s>]+)()/$1.&fix_cid($2,$_[1],$_[2]).$3/gei;
 return $rv;
 }
 
@@ -1146,25 +1325,31 @@ local ($quote, $html_edit, $body);
 local $cfg = defined(%userconfig) ? \%userconfig : \%config;
 local @writers = &split_addresses($mail->{'header'}->{'from'});
 local $writer = ($writers[0]->[1] || $writers[0]->[0])." wrote ..";
+local $qm = defined(%userconfig) ? $userconfig{'html_quote'}
+				 : $config{'html_quote'};
 if ($cfg->{'html_edit'} == 2 ||
     $cfg->{'html_edit'} == 1 && $htmlbody) {
 	# Create quoted body HTML
 	if ($htmlbody) {
 		$body = $htmlbody;
 		$sig =~ s/\n/<br>\n/g;
-		if ($qu) {
-			# Quoted HTML
-			$quote = "<html><body>\n".
-				 "$writer\n".
+		if ($qu && $qm == 0) {
+			# Quoted HTML as cite
+			$quote = "$writer\n".
 				 "<blockquote type=cite>\n".
 				 &safe_html($htmlbody->{'data'}).
-				 "</blockquote>".$sig."</body></html>\n";
+				 "</blockquote>".$sig."<br>\n";
+			}
+		elsif ($qu && $qm == 1) {
+			# Quoted HTML below line
+			$quote = "<br>$sig<hr>".
+			         "$writer<br>\n".
+				 &safe_html($htmlbody->{'data'});
 			}
 		else {
 			# Un-quoted HTML
-			$quote = "<html><body>\n".
-				 &safe_html($htmlbody->{'data'}).
-				 $sig."</body></html>\n";
+			$quote = &safe_html($htmlbody->{'data'}).
+				 $sig."<br>\n";
 			}
 		}
 	elsif ($plainbody) {
@@ -1172,19 +1357,23 @@ if ($cfg->{'html_edit'} == 2 ||
 		local $pd = $plainbody->{'data'};
 		$pd =~ s/^\s+//g;
 		$pd =~ s/\s+$//g;
-		if ($qu) {
-			# Quoted plain text as HTML
-			$quote = "<html><body>\n".
-				 "$writer\n".
+		if ($qu && $qm == 0) {
+			# Quoted plain text as HTML as cite
+			$quote = "$writer\n".
 				 "<blockquote type=cite>\n".
 				 "<pre>$pd</pre>".
-				 "</blockquote>".$sig."</body></html>\n";
+				 "</blockquote>".$sig."<br>\n";
+			}
+		elsif ($qu && $qm == 1) {
+			# Quoted plain text as HTML below line
+			$quote = "<br>$sig<hr>".
+				 "$writer<br>\n".
+				 "<pre>$pd</pre><br>\n";
 			}
 		else {
 			# Un-quoted plain text as HTML
-			$quote = "<html><body>\n".
-				 "<pre>$pd</pre>".
-				 $sig."</body></html>\n";
+			$quote = "<pre>$pd</pre>".
+				 $sig."<br>\n";
 			}
 		}
 	$html_edit = 1;
@@ -1239,5 +1428,52 @@ else {
 	}
 }
 
+# requires_delivery_notification(&mail)
+sub requires_delivery_notification
+{
+return $_[0]->{'header'}->{'disposition-notification-to'} ||
+       $_[0]->{'header'}->{'read-reciept-to'};
+}
+
+# send_delivery_notification(&mail, [from-addr], manual)
+# Send an email containing delivery status information
+sub send_delivery_notification
+{
+local ($mail, $from) = @_;
+$from ||= $mail->{'header'}->{'to'};
+local $host = &get_display_hostname();
+local $to = &requires_delivery_notification($mail);
+local $product = &get_product_name();
+$product = ucfirst($product);
+local $version = &get_webmin_version();
+local ($taddr) = &split_addresses($mail->{'header'}->{'to'});
+local $disp = $manual ? "manual-action/MDN-sent-manually"
+		      : "automatic-action/MDN-sent-automatically";
+local $dsn = <<EOF;
+Reporting-UA: $host; $product $version
+Original-Recipient: rfc822;$taddr->[0]
+Final-Recipient: rfc822;$taddr->[0]
+Original-Message-ID: $mail->{'header'}->{'message-id'}
+Disposition: $disp; displayed
+EOF
+local $dmail = {
+	'headers' =>
+	   [ [ 'From' => $from ],
+	     [ 'To' => $to ],
+	     [ 'Subject' => 'Delivery notification' ],
+	     [ 'Content-type' => 'multipart/report; report-type=disposition-notification' ],
+	     [ 'Content-Transfer-Encoding' => '7bit' ] ],
+	'attach' => [
+	   { 'headers' => [ [ 'Content-type' => 'text/plain' ] ],
+	     'data' => "This is a delivery status notification for the email sent to:\n$mail->{'header'}->{'to'}\non the date:\n$mail->{'header'}->{'date'}\nwith the subject:\n$mail->{'header'}->{'subject'}\n" },
+	   { 'headers' => [ [ 'Content-type' =>
+				'message/disposition-notification' ],
+			    [ 'Content-Transfer-Encoding' => '7bit' ] ],
+	     'data' => $dsn }
+		] };
+eval { local $main::errors_must_die = 1; &send_mail($dmail); };
+return $to;
+}
+
 1;
 
