@@ -8,7 +8,7 @@ do 'folders-lib.pl';
 &init_config();
 %access = &get_module_acl();
 $config{'perpage'} ||= 20;
-$gconfig{'logfiles'} = 0;	# file change logged never needs to be done
+$gconfig{'logfiles'} = 0;	# file change logging never needs to be done
 
 # Always detect the mail system if not set
 if ($config{'mail_system'} == 3) {
@@ -202,7 +202,13 @@ elsif ($access{'mmode'} == 4) {
 	return 1 if ($_[0] eq $remote_user);
 	}
 elsif ($access{'mmode'} == 5) {
-	return $u[3] eq $access{'musers'};
+	return 1 if ($u[3] eq $access{'musers'});
+	if ($access{'msec'}) {
+		# Check user's secondary groups too
+		local @ginfo = getgrgid($access{'musers'});
+		local @m = split(/\s+/, $ginfo[3]);
+		return 1 if (&indexof($_[0], @m) >= 0);
+		}
 	}
 elsif ($access{'mmode'} == 3) {
 	foreach $u (split(/\s+/, $access{'musers'})) {
@@ -220,16 +226,16 @@ elsif ($access{'mmode'} == 7) {
 return 0;	# can't happen!
 }
 
-# movecopy_user_select(number)
+# movecopy_user_select(number, folders, folder, form-no)
 # Returns HTML for entering a username to copy mail to
 sub movecopy_user_select
 {
 local $rv;
 $rv .= "<input type=submit name=move$_[0] value=\"$text{'mail_move'}\" ".
-       "onClick='return check_clicks(form)'>\n";
+       "onClick='return check_clicks(form)'>";
 $rv .= "<input type=submit name=copy$_[0] value=\"$text{'mail_copy'}\" ".
-       "onClick='return check_clicks(form)'>\n";
-$rv .= &ui_user_textbox("mfolder$_[0]");
+       "onClick='return check_clicks(form)'>";
+$rv .= &ui_user_textbox("mfolder$_[0]", undef, $_[3]);
 return $rv;
 }
 
@@ -367,7 +373,7 @@ return "<a href='list_mail.cgi?user=$_[0]&folder=$_[1]->{'index'}'>$text{'mail_r
 # Returns the address to use when sending email
 sub get_from_address
 {
-local $host = &get_system_hostname();
+local $host = &get_from_domain();
 if ($config{'webmin_from'} =~ /\@/) {
 	return $config{'webmin_from'};
 	}
@@ -379,17 +385,45 @@ else {
 	}
 }
 
+# get_from_domain()
+# Returns the default domain for From: addresses
+sub get_from_domain
+{
+return $config{'from_dom'} || &get_display_hostname();
+}
+
 # check_modification(&folder)
 # Display an error message if a folder has been modified since the time
 # in $in{'mod'}
 sub check_modification
 {
 local $newmod = &modification_time($_[0]);
-if ($in{'mod'} && $in{'mod'} != $newmod) {
+if ($in{'mod'} && $in{'mod'} != $newmod && $config{'check_mod'}) {
 	# Changed!
 	&error(&text('emodified', "list_mail.cgi?user=$in{'user'}&folder=$_[0]->{'index'}"));
 	}
 }
 
+# spam_report_cmd()
+# Returns a command for reporting spam, or undef if none
+sub spam_report_cmd
+{
+local %sconfig = &foreign_config("spam");
+if ($config{'spam_report'} eq 'sa_learn') {
+        return &has_command($sconfig{'sa_learn'}) ? "$sconfig{'sa_learn'} --spam
+" : undef;
+        }
+elsif ($config{'spam_report'} eq 'spamassassin') {
+        return &has_command($sconfig{'spamassassin'}) ? "$sconfig{'spamassassin'
+} --r" : undef;
+        }
+else {
+        return &has_command($sconfig{'sa_learn'}) ?
+                "$sconfig{'sa_learn'} --spam" :
+               &has_command($sconfig{'spamassassin'}) ?
+                "$sconfig{'spamassassin'} --r" : undef;
+        }
+}
+
 1;
 
