@@ -362,4 +362,26 @@ $rv .= "</select>\n";
 return $rv;
 }
 
+# restart_mon()
+# Re-start the MON process, returning undef on success or an error message
+# on failure
+sub restart_mon
+{
+if ($config{'restart_cmd'}) {
+        local $out =
+		&backquote_logged("$config{'restart_cmd'} 2>&1 </dev/null");
+        return "<tt>$out</tt>" if ($?);
+        }
+else {
+	local $pid = &check_pid_file($config{'pid_file'});
+	if ($pid) {
+                kill('HUP', $pid);
+		return undef;
+                }
+        else {
+                return $text{'restart_epid'};
+                }
+        }
+}
+
 1;
