@@ -32,16 +32,18 @@ foreach $db (@dbs) {
 	if (&supports_quoting()) {
 		$quoting = " --quote-names";
 		}
+	@tables = split(/\s+/, $config{'backup_tables_'.$db});
+	$tables = join(" ", map { quotemeta($_) } @tables);
 
-	&execute_before($db, STDOUT, 0, $file);
+	&execute_before($db, STDOUT, 0, $file, $all ? undef : $db);
 	unlink($file);
-	$out = &backquote_logged("$config{'mysqldump'} $authstr $drop $where $quoting ".quotemeta($db)." 2>&1 >$file");
+	$out = &backquote_logged("$config{'mysqldump'} $authstr $drop $where $quoting ".quotemeta($db)." $tables 2>&1 >$file");
 	if ($? || $out) {
 		print "Backup of database $db to file $file failed:\n";
 		print $out;
 		$ex = 1;
 		}
-	&execute_after($db, STDOUT, 0, $file);
+	&execute_after($db, STDOUT, 0, $file, $all ? undef : $db);
 	}
 exit($ex);
 
