@@ -26,17 +26,20 @@ foreach $db (@dbs) {
 	else {
 		$file = &date_subs($config{'backup_'.$db});
 		}
+	@tables = split(/\s+/, $config{'backup_tables_'.$db});
+	$tables = join(" ", map { " -t ".quotemeta($_) } @tables);
 	if (!$file) {
 		print STDERR "No backup file set for database $db\n";
 		exit(1);
 		}
 
-	&execute_before($db, STDOUT, 0, $file);
+	&execute_before($db, STDOUT, 0, $file, $all ? undef : $db);
 	unlink($file);
 	$bkup_command = $config{'dump_cmd'}.
 			($postgres_login ? " -U $postgres_login" : "").
 			($config{'host'} ? " -h $config{'host'}" : "").
 			($format eq 'p' ? "" : " -b").
+			$tables.
 			" -F$format -f ".quotemeta($file)." $db" ;
 
 	if ( $postgres_sameunix && defined(getpwnam($postgres_login)) ) {
@@ -55,7 +58,7 @@ foreach $db (@dbs) {
 		print STDERR $out;
 		$ex = 1;
 		}
-	&execute_after($db, STDOUT, 0, $file);
+	&execute_after($db, STDOUT, 0, $file, $all ? undef : $db);
 	}
 exit($ex);
 
