@@ -249,5 +249,19 @@ sub dialin_line
 return ($_[0]->{'not'} ? "!" : "").$_[0]->{'number'};
 }
 
+# apply_mgetty()
+# Apply the current serial port and mgetty configuration, or return an
+# error message
+sub apply_mgetty
+{
+local %iconfig = &foreign_config("inittab");
+local $out = &backquote_logged("$iconfig{'telinit'} q 2>&1 </dev/null");
+if ($?) {
+	return "<tt>$out</tt>";
+	}
+&kill_byname_logged("mgetty", 'TERM');
+return undef;
+}
+
 1;
 
