@@ -38,6 +38,9 @@ elsif ($action eq 'copy') {
 	return &text($type eq 'user' ? 'log_copy_u' : 'log_copy_g',
 		     "<tt>$object</tt>");
 	}
+elsif ($action eq 'email') {
+	return &text('log_email', "<tt>$object</tt>");
+	}
 else {
 	return undef;
 	}
