@@ -4,10 +4,17 @@
 do '../web-lib.pl';
 &init_config();
 require '../ui-lib.pl';
-do "$gconfig{'os_type'}-lib.pl";
+if ($gconfig{'os_type'} =~ /^\S+\-linux$/) {
+	do "linux-lib.pl";
+	}
+else {
+	do "$gconfig{'os_type'}-lib.pl";
+	}
 %access = &get_module_acl();
 &foreign_require("mount", "mount-lib.pl");
 
+$email_cmd = "$module_config_directory/email.pl";
+
 # list_filesystems()
 # Returns a list of details of local filesystems on which quotas are supported
 #  directory device type options quotacan quotanow
@@ -213,20 +220,32 @@ else {
 	}
 }
 
-# block_size(dir)
+# block_size(dir, [for-filesys])
 # Returns the size (in bytes) of blocks on some filesystem
 sub block_size
 {
 return undef if (!$config{'block_mode'});
-return undef if (!defined(&fs_block_size));
+return undef if (!defined(&quota_block_size) &&
+		 !defined(&fs_block_size));
 local @mounts = &mount::list_mounted();
 local ($mount) = grep { $_->[0] eq $_[0] } @mounts;
 if ($mount) {
-	return &fs_block_size(@$mount);
+	if ($_[1]) {
+		return &fs_block_size(@$mount);
+		}
+	else {
+		if (defined(&quota_block_size)) {
+			return &quota_block_size(@$mount);
+			}
+		else {
+			return &fs_block_size(@$mount);
+			}
+		}
 	}
 return undef;
 }
 
+# print_limit(amount, no-blocks)
 sub print_limit
 {
 if ($_[0] == 0) { print "<td>$text{'quota_unlimited'}</td>\n"; }
@@ -234,5 +253,27 @@ elsif ($bsize && !$_[1]) { print "<td>",&nice_size($_[0]*$bsize),"</td>"; }
 else { print "<td>$_[0]</td>\n"; }
 }
 
+sub print_grace
+{
+print "<td>",($_[0] || "<br>"),"</td>\n";
+}
+
+sub find_email_job
+{
+local @jobs = &cron::list_cron_jobs();
+local ($job) = grep { $_->{'command'} eq $email_cmd } @jobs;
+return $job;
+}
+
+# trunc_space(string)
+# Removes spaces from the start and end of a string
+sub trunc_space
+{
+local $rv = $_[0];
+$rv =~ s/^\s+//;
+$rv =~ s/\s+$//;
+return $rv;
+}
+
 1;
 
