@@ -157,16 +157,20 @@ local($l, $started, @rv);
 if ($samba_version >= 3) {
 	# New samba status format
 	local %pidmap;
-	foreach $l (split(/\n/ , `$config{samba_status_program} -S`)) {
+	foreach $l (split(/\n/ , `$config{samba_status_program}`)) {
 		if ($l =~ /^----/) {
 			$started++;
 			}
 		elsif ($started == 1 &&
 		       $l =~ /^\s*(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\((\S+)\)/) {
+			# A line with a PID, username, group and hostname
 			$pidmap{$1} = [ $1, $2, $3, $4, $5 ];
 			}
 		elsif ($started == 2 &&
 		       $l =~ /^\s*(\S+)\s+(\d+)\s+(\S+)\s+(.*)$/) {
+			# A line with a service, PID and machine. This must be
+			# combined data from the first type of line to create
+			# the needed information
 			local $p = $pidmap{$2};
 			if (!$_[0] || $1 eq $_[0] ||
 			    $1 eq $p->[1] && $_[0] eq "homes") {
@@ -347,7 +351,9 @@ return $default_values{$_[0]};
 		"lock directory,lock dir",
 		"max xmit,max packet",
 		"root directory,root dir,root",
-		"case sensitive,case sig names"
+		"case sensitive,case sig names",
+		"idmap uid,winbind uid",
+		"idmap gid,winbind gid",
 	 );
 foreach $s (@synon) {
 	foreach $ss (split(/,/ , $s)) {
@@ -426,7 +432,13 @@ foreach $s (@synon) {
 			"wide links","yes",
 			"wins support","no",
 			"writable","no",
-			"write list",undef );
+			"write list",undef,
+			"winbind cache time",300,
+			"winbind enable local accounts","yes",
+			"winbind trusted domains only","no",
+			"winbind enum users","yes",
+			"winbind enum groups","yes",
+		);
 $default_values{'encrypt passwords'} = 'yes' if ($samba_version >= 3);
 
 # user_list(list)
