@@ -24,6 +24,9 @@ else {
 	elsif (&has_command("up2date") && &has_command("rhn_check")) {
 		$update_system = "rhn";
 		}
+	elsif (-x "/opt/csw/bin/pkg-get" || &has_command("pkg-get")) {
+		$update_system = "csw";
+		}
 	elsif (&has_command("cupdate")) {
 		# not done yet!
 		}
