@@ -249,7 +249,7 @@ printf "<input type=radio name=$_[0]_def value=1 %s> %s %s\n",
 printf "<input type=radio name=$_[0]_def value=0 %s>\n",
 	$_[1] ? "checked" : "";
 printf "<input name=$_[0] size=$_[2] value='%s'>\n",
-	$_[1] ? &html_escape($_[1]->{'value'}) : "";
+	$_[1] ? &html_escape(ref($_[1]) ? $_[1]->{'value'} : $_[1]) : "";
 }
 
 # parse_opt(&config, name, [&checkfunc])
@@ -336,10 +336,19 @@ sub can_use_check
 sub get_spamassassin_version
 {
 local $out = `$config{'spamassassin'} -V 2>&1`;
-${$_[0]} = $out;
+${$_[0]} = $out if ($_[0]);
 return $out =~ /(version|Version:)\s+(\S+)/ ? $2 : undef;
 }
 
+# version_atleast(num)
+sub version_atleast
+{
+if (!$version_cache) {
+	$version_cache = &get_spamassassin_version();
+	}
+return $version_cache >= $_[0];
+}
+
 # spam_file_folder()
 sub spam_file_folder
 {
@@ -415,6 +424,27 @@ print "<td align=right><input type=submit name=razor value=\"$text{'mail_razor'}
 print "</tr></table>\n";
 }
 
+# restart_spamd()
+# Re-start all SpamAssassin processes, or return an error message
+sub restart_spamd
+{
+if ($config{'restart_cmd'}) {
+	local $out = &backquote_logged(
+		"$config{'restart_cmd'} 2>&1 </dev/null");
+	if ($? || $out =~ /error|failed/i) {
+		return "<pre>$out</pre>";
+		}
+	}
+else {
+	local @pids = &get_process_pids();
+	@pids || return $text{'apply_none'};
+	local $p;
+	foreach $p (@pids) {
+		&kill_logged("HUP", $p->[0]);
+		}
+	}
+return undef;
+}
 
 1;
 
