@@ -17,7 +17,8 @@ elsif ($type eq 'host') {
 	}
 elsif ($type eq 'http' || $type eq 'icp' ||
        $type eq 'always' || $type eq 'never' ||
-       $type eq 'pool' || $type eq 'delay' || $type eq 'headeracc') {
+       $type eq 'pool' || $type eq 'delay' ||
+       $type eq 'headeracc' || $type eq 'refresh') {
 	return &text("log_${type}_${action}",
 		     "<tt>".&html_escape($object)."</tt>");
 	}
