@@ -355,5 +355,25 @@ foreach $k (keys %{$_[0]->{'values'}}) {
 return @rv;
 }
 
+# apply_configuration()
+# Apply the inetd and/or xinetd configuration
+sub apply_configuration
+{
+if ($has_inetd) {
+	local %iconfig = &foreign_config("inetd");
+	&system_logged("$iconfig{'restart_command'} >/dev/null 2>&1 </dev/null");
+	}
+if ($has_xinetd) {
+	local %xconfig = &foreign_config("xinetd");
+	local $pid;
+	if (open(PID, $xconfig{'pid_file'})) {
+		chop($pid = <PID>);
+		close(PID);
+		kill('USR2', $pid);
+		}
+	}
+return undef;
+}
+
 1;
 
