@@ -194,5 +194,16 @@ else {
 	}
 }
 
+# apply_configuration()
+# Apply the vgetty serial port configuration. Returns undef on success, or an
+# error message on failure
+sub apply_configuration
+{
+local $out = &backquote_logged("telinit q 2>&1 </dev/null");
+return "<tt>$out</tt>" if ($?);
+&system_logged("killall vgetty");
+return undef;
+}
+
 1;
 
