@@ -241,5 +241,50 @@ ${$_[0]} = $out;
 return $out =~ /\sV(\S+)/ ? $1 : undef;
 }
 
+# get_all_logs()
+# Returns a list of all log files the module can report on
+sub get_all_logs
+{
+# Query apache and squid for their logfiles
+local %auto = map { $_, 1 } split(/,/, $config{'auto'});
+local @logs;
+if (&foreign_check("apache") && $auto{'apache'}) {
+	&foreign_require("apache", "apache-lib.pl");
+	local $conf = &apache::get_config();
+	local @dirs = ( &apache::find_all_directives($conf, "CustomLog"),
+		  	&apache::find_all_directives($conf, "TransferLog") );
+	local $root = &apache::find_directive_struct("ServerRoot", $conf);
+	local $d;
+	foreach $d (@dirs) {
+		local $lf = $d->{'words'}->[0];
+		next if ($lf =~ /^\|/);
+		if ($lf !~ /^\//) {
+			$lf = "$root->{'words'}->[0]/$lf";
+			}
+		open(FILE, $lf);
+		local $line = <FILE>;
+		close(FILE);
+		if (!$line || $line =~ /^([a-zA-Z0-9\.\-]+)\s+\S+\s+\S+\s+\[\d+\/[a-zA-z]+\/\d+:\d+:\d+:\d+\s+[0-9\+\-]+\]/) {
+			push(@logs, { 'file' => $lf,
+				      'type' => 1 });
+			}
+		}
+	}
+if (&foreign_check("squid") && $auto{'squid'}) {
+	&foreign_require("squid", "squid-lib.pl");
+	local $conf = &squid::get_config();
+	local $log = &squid::find_value("cache_access_log", $conf);
+	$log = "$squid::config{'log_dir'}/access.log"
+		if (!$log && -d $squid::config{'log_dir'});
+	push(@logs, { 'file' => $log,
+		      'type' => 2 }) if ($log);
+	}
+
+# Add custom logfiles
+push(@logs, map { $_->{'custom'} = 1; $_ } &read_custom_logs());
+
+return @logs;
+}
+
 1;
 
